interface ScalerInterface {
    fit(X: number[][]): void;
    transform(X: number[][]): number[][];
    fitTransform(X: number[][]): number[][];
}
interface ModelInterface {
    fit(X: number[][], y: number[]): void;
    predict(X: number[][]): number[];
    score(X: number[][], y: number[]): number;
}

/**
 * Linear Regression model implementation using ordinary least squares
 * @class LinearRegression
 * @implements {ModelInterface}
 */
declare class LinearRegression implements ModelInterface {
    private weights;
    /**
     * Fits the linear regression model to the training data
     * @param {number[][]} X - Training data features matrix
     * @param {number[]} y - Training data target values
     * @throws {DataValidationError} If input dimensions don't match or data is empty
     */
    fit(X: number[][], y: number[]): void;
    /**
     * Make predictions using the trained model
     * @param X Prediction data matrix, each row represents a sample, each column represents a feature
     * @returns Vector of predictions
     * @throws {DataValidationError} If model is not trained or input dimensions are incorrect
     */
    predict(X: number[][]): number[];
    /**
     * Solve linear system Ax = b
     */
    private solve;
    score(X: number[][], y: number[]): number;
}

declare class StandardScaler implements ScalerInterface {
    private mean;
    private std;
    /**
     * Calculate the mean and standard deviation of features
     */
    fit(X: number[][]): void;
    /**
     * Standardize data using the calculated mean and standard deviation
     */
    transform(X: number[][]): number[][];
    /**
     * Combine fit and transform operations
     */
    fitTransform(X: number[][]): number[][];
}

declare class MinMaxScaler implements ScalerInterface {
    private min;
    private max;
    /**
     * Fit the scaler to the data
     * @param {number[][]} X - Input data matrix
     */
    fit(X: number[][]): void;
    /**
     * Fit the scaler to the data and transform it
     * @param X Input data matrix
     * @returns Scaled data matrix
     */
    fitTransform(X: number[][]): number[][];
    /**
     * Transform data using the fitted scaler
     * @param X Input data matrix
     * @returns Scaled data matrix
     */
    transform(X: number[][]): number[][];
}

declare function trainTestSplit<T>(data: T[], testSize?: number, randomSeed?: number): {
    train: T[];
    test: T[];
};

interface PipelineResult {
    model: LinearRegression;
    predictions: number[];
    trueValues: number[];
    scaler?: StandardScaler | MinMaxScaler;
    testFeaturesRaw?: Dataset;
    testFeaturesScaled?: Dataset;
    trainingScore?: number;
}
type Dataset = number[][];

/**
 * Creates and trains a machine learning pipeline with data preprocessing and model training
 * @param X Feature matrix
 * @param y Target vector
 * @param testSize Proportion of data to use for testing (default: 0.2)
 * @returns PipelineResult containing trained model, predictions and true values
 */
declare function createPipelineAndTrain(X: Dataset, y: number[], testSize?: number, preprocess?: "standard" | "minmax", randomSeed?: number): PipelineResult;

export { type Dataset, LinearRegression, MinMaxScaler, type PipelineResult, StandardScaler, createPipelineAndTrain, trainTestSplit };
