"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  LinearRegression: () => LinearRegression,
  MinMaxScaler: () => MinMaxScaler,
  StandardScaler: () => StandardScaler,
  createPipelineAndTrain: () => createPipelineAndTrain,
  trainTestSplit: () => trainTestSplit
});
module.exports = __toCommonJS(src_exports);

// src/models/LinearRegression.ts
var math = __toESM(require("mathjs"));
var import_numeric = __toESM(require("numeric"));

// src/exceptions/index.ts
var MLError = class extends Error {
  constructor(message) {
    super(message);
    this.name = "MLError";
  }
};
var DataValidationError = class extends MLError {
  constructor(message) {
    super(message);
    this.name = "DataValidationError";
  }
};

// src/models/LinearRegression.ts
var LinearRegression = class {
  constructor() {
    this.weights = null;
  }
  /**
   * Fits the linear regression model to the training data
   * @param {number[][]} X - Training data features matrix
   * @param {number[]} y - Training data target values
   * @throws {DataValidationError} If input dimensions don't match or data is empty
   */
  fit(X, y) {
    if (X.length !== y.length) {
      throw new DataValidationError("Number of samples in X and y must match");
    }
    if (X.length === 0) {
      throw new DataValidationError("Empty training data");
    }
    const featureCount = X[0].length;
    if (!X.every((row) => row.length === featureCount)) {
      throw new DataValidationError("Inconsistent number of features in X");
    }
    const X_transpose = math.transpose(math.matrix(X));
    const XTX = math.multiply(X_transpose, X);
    const XTy = math.multiply(X_transpose, math.matrix(y.map((val) => [val])));
    this.weights = this.solve(XTX, XTy);
  }
  /**
   * Make predictions using the trained model
   * @param X Prediction data matrix, each row represents a sample, each column represents a feature
   * @returns Vector of predictions
   * @throws {DataValidationError} If model is not trained or input dimensions are incorrect
   */
  predict(X) {
    if (!this.weights) {
      throw new DataValidationError("Model not trained yet");
    }
    if (X.length === 0) {
      throw new DataValidationError("Empty prediction data");
    }
    if (!X.every((row) => row.length === this.weights.size()[0])) {
      throw new DataValidationError("Number of features must match training data");
    }
    return X.map((row) => math.dot(row, this.weights.toArray()));
  }
  /**
   * Solve linear system Ax = b
   */
  solve(A, b) {
    const { U, S, V } = import_numeric.default.svd(A.toArray());
    const S_diag = math.matrix(math.zeros(A.size()[0], A.size()[1]));
    for (let i = 0; i < S.length; i++) {
      S_diag.set([i, i], S[i]);
    }
    const S_end = math.matrix(S_diag);
    const S_inv = math.inv(S_end);
    const V_T = math.transpose(V);
    const X = math.multiply(V_T, math.multiply(S_inv, math.multiply(U, b)));
    return X;
  }
  score(X, y) {
    const predictions = this.predict(X);
    const meanY = y.reduce((sum, val) => sum + val, 0) / y.length;
    let totalSS = 0;
    let residualSS = 0;
    for (let i = 0; i < y.length; i++) {
      totalSS += Math.pow(y[i] - meanY, 2);
      residualSS += Math.pow(y[i] - predictions[i], 2);
    }
    if (totalSS === 0) return 0;
    return 1 - residualSS / totalSS;
  }
};

// src/preprocessing/StandardScaler.ts
var math2 = __toESM(require("mathjs"));
var StandardScaler = class {
  constructor() {
    this.mean = [];
    this.std = [];
  }
  /**
   * Calculate the mean and standard deviation of features
   */
  fit(X) {
    if (!X || !X.length || !X[0].length) {
      throw new Error("Input array cannot be empty");
    }
    const nFeatures = X[0].length;
    const nSamples = X.length;
    this.mean = new Array(nFeatures).fill(0);
    this.std = new Array(nFeatures).fill(0);
    for (let i = 0; i < nFeatures; i++) {
      this.mean[i] = math2.mean(X.map((row) => row[i]));
    }
    for (let i = 0; i < nFeatures; i++) {
      this.std[i] = Number(
        math2.std(
          X.map((row) => row[i]),
          "uncorrected"
        )
      );
      if (this.std[i] === 0) {
        this.std[i] = 1;
      }
    }
  }
  /**
   * Standardize data using the calculated mean and standard deviation
   */
  transform(X) {
    if (!X || !X.length || !X[0].length) {
      throw new Error("Input array cannot be empty");
    }
    if (X[0].length !== this.mean.length) {
      throw new Error("Feature dimensions do not match training set");
    }
    return X.map(
      (row) => row.map((value, i) => (value - this.mean[i]) / this.std[i])
    );
  }
  /**
   * Combine fit and transform operations
   */
  fitTransform(X) {
    this.fit(X);
    return this.transform(X);
  }
};

// src/preprocessing/MinMaxScaler.ts
var math3 = __toESM(require("mathjs"));
var MinMaxScaler = class {
  constructor() {
    this.min = null;
    this.max = null;
  }
  /**
   * Fit the scaler to the data
   * @param {number[][]} X - Input data matrix
   */
  fit(X) {
    if (X.length === 0) {
      throw new Error("Empty input data");
    }
    const featureCount = X[0].length;
    if (!X.every((row) => row.length === featureCount)) {
      throw new Error("Inconsistent number of features");
    }
    this.min = new Array(featureCount).fill(Infinity);
    this.max = new Array(featureCount).fill(-Infinity);
    for (let i = 0; i < X.length; i++) {
      for (let j = 0; j < featureCount; j++) {
        this.min[j] = math3.min(this.min[j], X[i][j]);
        this.max[j] = math3.max(this.max[j], X[i][j]);
      }
    }
  }
  /**
   * Fit the scaler to the data and transform it
   * @param X Input data matrix
   * @returns Scaled data matrix
   */
  fitTransform(X) {
    if (X.length === 0) {
      throw new Error("Empty input data");
    }
    const featureCount = X[0].length;
    if (!X.every((row) => row.length === featureCount)) {
      throw new Error("Inconsistent number of features");
    }
    this.min = new Array(featureCount).fill(Infinity);
    this.max = new Array(featureCount).fill(-Infinity);
    for (let i = 0; i < X.length; i++) {
      for (let j = 0; j < featureCount; j++) {
        this.min[j] = math3.min(this.min[j], X[i][j]);
        this.max[j] = math3.max(this.max[j], X[i][j]);
      }
    }
    return this.transform(X);
  }
  /**
   * Transform data using the fitted scaler
   * @param X Input data matrix
   * @returns Scaled data matrix
   */
  transform(X) {
    if (!this.min || !this.max) {
      throw new Error("Scaler not fitted yet");
    }
    if (X.length === 0) {
      throw new Error("Empty input data");
    }
    if (!X.every((row) => row.length === this.min.length)) {
      throw new Error("Number of features must match training data");
    }
    return X.map(
      (row) => row.map((val, j) => {
        const range = math3.subtract(this.max[j], this.min[j]);
        return range === 0 ? 0 : math3.divide(math3.subtract(val, this.min[j]), range);
      })
    );
  }
};

// src/validation/trainTestSplit.ts
function trainTestSplit(data, testSize = 0.2, randomSeed) {
  if (!data || data.length === 0) {
    throw new Error("Input data cannot be empty");
  }
  if (testSize <= 0 || testSize >= 1) {
    throw new Error("Test size must be between 0 and 1");
  }
  const testCount = Math.floor(data.length * testSize);
  if (testCount === 0) {
    throw new Error("Test size too small, resulting in empty test set");
  }
  const shuffled = [...data];
  if (randomSeed !== void 0) {
    let seed = randomSeed;
    for (let i = shuffled.length - 1; i > 0; i--) {
      seed = (seed * 9301 + 49297) % 233280;
      const j = Math.floor(seed / 233280 * (i + 1));
      [shuffled[i], shuffled[j]] = [shuffled[j], shuffled[i]];
    }
  } else {
    for (let i = shuffled.length - 1; i > 0; i--) {
      const j = Math.floor(Math.random() * (i + 1));
      [shuffled[i], shuffled[j]] = [shuffled[j], shuffled[i]];
    }
  }
  const trainSize = shuffled.length - testCount;
  return {
    train: shuffled.slice(0, trainSize),
    test: shuffled.slice(trainSize)
  };
}

// src/index.ts
function createPipelineAndTrain(X, y, testSize = 0.2, preprocess = "standard", randomSeed) {
  if (!Array.isArray(X) || !Array.isArray(y)) {
    throw new Error("Input data must be arrays");
  }
  if (!X.every(
    (row) => Array.isArray(row) && row.every((val) => typeof val === "number")
  )) {
    throw new Error("Feature matrix X must be a 2D array of numbers");
  }
  if (!y.every((val) => typeof val === "number")) {
    throw new Error("Target vector y must be an array of numbers");
  }
  const { train: X_train, test: X_test } = trainTestSplit(
    X,
    testSize,
    randomSeed
  );
  const { train: y_train, test: y_test } = trainTestSplit(
    y,
    testSize,
    randomSeed
  );
  const scaler = preprocess === "standard" ? new StandardScaler() : new MinMaxScaler();
  const X_train_scaled = scaler.fitTransform(X_train);
  const X_test_scaled = scaler.transform(X_test);
  if (X_train_scaled.length === 0) {
    throw new Error("Training data is empty after preprocessing");
  }
  if (X_test_scaled.length === 0) {
    throw new Error("Testing data is empty after preprocessing");
  }
  const model = new LinearRegression();
  model.fit(X_train_scaled, y_train);
  const predictions = model.predict(X_test_scaled);
  return {
    model,
    predictions,
    trueValues: y_test,
    scaler,
    testFeaturesRaw: X_test,
    testFeaturesScaled: X_test_scaled,
    trainingScore: model.score ? model.score(X_train_scaled, y_train) : void 0
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  LinearRegression,
  MinMaxScaler,
  StandardScaler,
  createPipelineAndTrain,
  trainTestSplit
});
